import React, { Fragment } from 'react';
import { createPortal } from 'react-dom';
import { observable } from '@formily/reactive';
import { Observer } from '@formily/react';
import { render as reactRender, unmount as reactUnmount } from './render';
var PortalMap = observable(new Map());
export var createPortalProvider = function (id) {
    var Portal = function (props) {
        var _a;
        var portalId = (_a = props.id) !== null && _a !== void 0 ? _a : id;
        if (portalId && !PortalMap.has(portalId)) {
            PortalMap.set(portalId, null);
        }
        return (React.createElement(Fragment, null,
            props.children,
            React.createElement(Observer, null, function () {
                if (!portalId)
                    return null;
                var portal = PortalMap.get(portalId);
                if (portal)
                    return createPortal(portal, document.body);
                return null;
            })));
    };
    return Portal;
};
export function createPortalRoot(host, id) {
    function render(renderer) {
        if (PortalMap.has(id)) {
            PortalMap.set(id, renderer === null || renderer === void 0 ? void 0 : renderer());
        }
        else if (host) {
            reactRender(React.createElement(Fragment, null, renderer === null || renderer === void 0 ? void 0 : renderer()), host);
        }
    }
    function unmount() {
        var _a;
        if (PortalMap.has(id)) {
            PortalMap.set(id, null);
        }
        if (host) {
            var unmountResult = reactUnmount(host);
            if (unmountResult && host.parentNode) {
                (_a = host.parentNode) === null || _a === void 0 ? void 0 : _a.removeChild(host);
            }
        }
    }
    return {
        render: render,
        unmount: unmount,
    };
}
//# sourceMappingURL=portal.js.map